@@ -12,7 +12,17 @@ gowerD <- function(data.x, data.y = data.x, weights=NULL,numerical,factors,order
   weightind <- order(match(colnames(data.x),c(numerical,factors,orders,mixed)))
   data.x <- data.x[,c(numerical,factors,orders,mixed),drop=FALSE]
   data.y <- data.y[,c(numerical,factors,orders,mixed),drop=FALSE]
-  
+  if(length(numerical)>0){
+    ##Datensatz durch Range dividieren
+    rmin <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,min,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,min,na.rm=TRUE)),2,min,na.rm=TRUE)
+    rmax <- apply(rbind(apply(data.x[,numerical,drop=FALSE],2,max,na.rm=TRUE),apply(data.y[,numerical,drop=FALSE],2,max,na.rm=TRUE)),2,max,na.rm=TRUE)
+    r <- rmax-rmin
+    r[r==0] <- 1
+    for(i in seq_along(numerical)){
+      data.x[,numerical[i]] <- data.x[,numerical[i]]/r[i]
+      data.y[,numerical[i]] <- data.y[,numerical[i]]/r[i]
+    }
+  }
   justone <- FALSE
   if(nrow(data.y)==1){
     data.y <- rbind(data.y,data.y)
